using System;
using System.Collections.Generic;
using System.Text;
using NationalInstruments.ModularInstruments.Interop;

namespace RFSAGettingStartedIQ
{
    class Program
    {
        static void Main(string[] args)
        {
            int numberOfSamples = 1000;
            Double magnitudeSquared;
            Double accumulator = 0.0;
            niComplexNumber[] data;
            niRFSA_wfmInfo wfmInfo;
            
            try
            {
                // Initialize a session to the NI-RFSA device
                using (niRFSA rfsa = new niRFSA("PXI1Slot2", true, false))
                {
                    // Configure NI-RFSA for a simple IQ acquisition 
                    rfsa.ConfigureRefClock("OnboardClock", 10e6);
                    rfsa.ConfigureReferenceLevel("", 0);
                    rfsa.ConfigureAcquisitionType(niRFSAConstants.Iq);
                    rfsa.ConfigureIQCarrierFrequency("", 1e9);
                    rfsa.ConfigureNumberOfSamples("", true, numberOfSamples);
                    rfsa.ConfigureIQRate("", 1e6);
                    
                    // Allocate an array of right size for the driver to fill-in data
                    data = new niComplexNumber[numberOfSamples];
                    rfsa.ReadIQSingleRecordComplexF64("", 10.0, data, numberOfSamples, out wfmInfo);

                    // Do something useful with the data
                    // We will present average power: 10log(((I^2 + Q ^2) / 2R) * 1000), where R = 50 Ohms
                    if (numberOfSamples > 0)
                    {
                        for (int i = 0; i < numberOfSamples; ++i)
                        {
                            magnitudeSquared = data[i].Real * data[i].Real + data[i].Imaginary * data[i].Imaginary;
                            if (magnitudeSquared == 0.0)
                            {
                                magnitudeSquared = 0.00000001;
                            }
                            accumulator += 10.0 * Math.Log10((magnitudeSquared / (2.0 * 50.0)) * 1000.0);
                        }
                    }
                } // rfsa session will automatically be closed here

                Console.WriteLine("Average power = " + accumulator / numberOfSamples + " dBm" + Environment.NewLine);
            }
            catch (System.Runtime.InteropServices.ExternalException e)
            {
                Console.WriteLine("ERROR : " + e.Message.ToString());
            }
            Console.ReadKey();
        }
    }
}
