Imports NationalInstruments.ModularInstruments.Interop
Module MainModule

    Sub Main()
        Dim numberOfSpectralLines As Integer
        Dim spectrumData As Double() = Nothing
        Dim greatestPeakPower As Double = 0
        Dim greatestPeakFrequency As Double = 0
        Dim spectrumInfo As niRFSA_spectrumInfo

        Try
            ' Initialize a session to the NI-RFSA device
            Using rfsa As niRFSA = New niRFSA("PXI1Slot2", True, False)

                ' Configure NI-RFSA for a simple IQ acquisition 
                rfsa.ConfigureRefClock("OnboardClock", 10000000)
                rfsa.ConfigureReferenceLevel("", 0)
                rfsa.ConfigureAcquisitionType(niRFSAConstants.Spectrum)
                rfsa.ConfigureSpectrumFrequencyStartStop("", 990000000.0, 1010000000.0)
                rfsa.ConfigureResolutionBandwidth("", 10000.0)

                ' Allocate an array of right size for the driver to fill-in the power spectrum data
                rfsa.GetNumberOfSpectralLines("", numberOfSpectralLines)
                spectrumData = New Double(numberOfSpectralLines - 1) {}
                rfsa.ReadPowerSpectrumF64("", 10, spectrumData, numberOfSpectralLines, spectrumInfo)

                ' Do something useful with the data
                ' We will find the highest peak in a bin, which is not the actual highest
                ' peak and frequency we could find in the acquisition. For an accurate
                ' peak search, we can analyze the data with the Spectral Measurements Toolset.
                Dim i As Integer
                For i = 0 To numberOfSpectralLines - 1
                    If ((i = 0) OrElse (spectrumData(i) > greatestPeakPower)) Then
                        greatestPeakPower = spectrumData(i)
                        greatestPeakFrequency = (spectrumInfo.initialFrequency + (spectrumInfo.frequencyIncrement * i))
                    End If
                Next i
            End Using ' rfsa session will automatically be closed here

            Console.WriteLine(("The highest peak in a bin is " & greatestPeakPower & "dBm at " & (greatestPeakFrequency / 1000000) & " MHz." & Environment.NewLine))
        Catch e As System.Runtime.InteropServices.ExternalException
            Console.WriteLine(("ERROR : " & e.Message.ToString))
        End Try
        Console.ReadKey()
    End Sub

End Module
